<?php

namespace FiloBlu\Flow\Model\From\Entity;

/**
 * Class Product
 * @package FiloBlu\Flow\Model\From\Entity
 */
class Product extends AbstractEntity
{

    /**
     * @return bool|void
     * @throws \Exception
     */
    protected function setObjectAttributeValue()
    {
        if (!$this->getObjectId()) {
            throw new \Exception(sprintf('No id found for model %s with given identifier %s', $this->getEntityModel(), $this->entityIdentifier));
        }

        $attribute_code = $this->getAttribute()->getAttributeCode();

        $pids = [$this->getObjectId()];

        $attributesToCopy = $this->registry->registry('attribute_configurable_copy');

        $fatherProductsIds = $this->helperProduct->getParentProductsIds($pids);

        if (!empty($fatherProductsIds) && in_array($attribute_code, $attributesToCopy)) {
            $pids = array_merge($pids, $fatherProductsIds);
        }

        $attributeValue = $this->getAttributeValue();

        $attributesData = [$attribute_code => $attributeValue];

        if ($attribute_code != 'sku') {
            \Magento\Framework\App\ObjectManager::getInstance()->get('Magento\Catalog\Model\Product\Action')
                ->updateAttributes($pids, $attributesData, $this->storeId);
        }

    }

    /**
     * @return bool|mixed|null
     * @throws \Exception
     */
    public function getObjectId()
    {
        if (is_null($this->_objectEntityId)) {
            $this->_objectEntityId = \Magento\Framework\App\ObjectManager::getInstance()->create($this->getEntityModel())->getIdBySku($this->entityIdentifier);
            if (empty($this->_objectEntityId)) {
                $this->_objectEntityId = false;
            }
        }

        return $this->_objectEntityId;
    }

    /**
     * @return |null
     * @throws \Exception
     */
    protected function getObject()
    {
        if (is_null($this->objectEntity)) {
            if (!$this->getObjectId()) {
                throw new \Exception('No id found for model ' . $this->getEntityModel() . ' with given identifier ' . $this->entityIdentifier);
            }
            $_prod_sku = $this->productFactory->create()->getIdBySku($this->entityIdentifier);
            $this->objectEntity = $this->productFactory->create()->setStoreId($this->storeId)->load($_prod_sku);
            if (!$this->objectEntity->getId()) {
                throw new \Exception('No instance found for model ' . $this->getEntityModel() . ' with id ' . $this->getObjectId());
            }
        }
        return $this->objectEntity;
    }

}
