<?php

namespace FiloBlu\Flow\Model\From;

/**
 * Class Stock
 * @package FiloBlu\Flow\Model\From
 */
class Stock extends AbstractFrom {

    /**
     * @var \Monolog\Logger
     */
    protected $_logger;
    /**
     * @var \FiloBlu\Flow\Helper\Data
     */
    protected $_helper_flow;
    /**
     * @var \Magento\Catalog\Model\ProductFactory
     */
    protected $_product_factory;
    /**
     * @var \Magento\CatalogInventory\Api\StockRegistryInterface
     */
    protected $_stock_registry;
    /**
     * @var \FiloBlu\Flow\Helper\Stock
     */
    protected $_stock_qty;
    /**
     * @var \Magento\Framework\App\ProductMetadataInterface
     */
    protected $_productMetadata;

    /**
     * @var \FiloBlu\Flow\Helper\EmailNotification
     */
    protected $_emailnotification_helper;

    /**
     * @var \Magento\Framework\App\Config\ScopeConfigInterface
     */
    protected $_scopeConfig;

    /**
     * Stock constructor.
     * @param \Magento\Framework\Model\Context $context
     * @param \Magento\Framework\Registry $registry
     * @param \FiloBlu\Flow\Helper\Data $helper_flow
     * @param \FiloBlu\Flow\Helper\LoggerProvider $loggerProvider
     * @param \Magento\Catalog\Model\ProductFactory $productFactory
     * @param \Magento\CatalogInventory\Api\StockRegistryInterface $stockRegistry
     * @param \FiloBlu\Flow\Helper\Stock $stock_qty
     * @param \Magento\Framework\App\ProductMetadataInterface $productMetadata
     * @param array $data
     */
    public function __construct(
        \Magento\Framework\Model\Context $context,
        \Magento\Framework\Registry $registry,
        \FiloBlu\Flow\Helper\Data $helper_flow,
        \FiloBlu\Flow\Helper\LoggerProvider $loggerProvider,
        \Magento\Catalog\Model\ProductFactory $productFactory,
        \Magento\CatalogInventory\Api\StockRegistryInterface $stockRegistry,
        \FiloBlu\Flow\Helper\Stock $stock_qty,
        \Magento\Framework\App\ProductMetadataInterface $productMetadata,
        \FiloBlu\Flow\Helper\EmailNotification $emailnotification_helper,
        \Magento\Framework\App\Config\ScopeConfigInterface $scopeConfig,
        array $data = []
    ) {
        $this->_helper_flow = $helper_flow;
        $this->_logger = $loggerProvider->getLogger();
        $this->_product_factory = $productFactory;
        $this->_stock_registry = $stockRegistry;
        $this->_stock_qty= $stock_qty;
        $this->_productMetadata = $productMetadata;
        $this->_emailnotification_helper= $emailnotification_helper;
        $this->_scopeConfig = $scopeConfig;
        parent::__construct($context, $registry, null, null, $data);
    }

    /**
     * @return void
     */
    public function _construct()
    {
        $this->_init('FiloBlu\Flow\Model\ResourceModel\From\Stock');
    }

    /**
     * @return $this|bool|mixed
     * @throws \Exception
     */
    public function process(){



        //  $this->_logger->info(__METHOD__);

        $product_id = $this->_product_factory->create()->getIdBySku($this->getSku());

        //$product = $this->_product_factory->create()->setStoreId(0);

        if(!$product_id){
            $message = sprintf("Product not found! product_code: '%s'", $this->getSku());
            return $this->error(false, $message);
        }

        if($product_id){

            // salvo il prezzo sul prodotto semplice
            //$product->load($product_id);

            $stock = $this->_stock_registry->getStockItem($product_id);

            $_in_stock = false;

            //$this->_logger->info('Process STOCK ---> ' . $product_id);

            try {

                $raw_qty = intval($this->getData('fisico_disponibile'));

                $qty = $this->_stock_qty->getRealQty($product_id,$raw_qty);
                
                // Se non viene ritornata una quantità numerica ma una stringa vuol dire che c'è stato un errore
                if (!is_numeric($qty)){
                    // Salvo il messaggio relativo all'errore
                    $this->addEmailNotifyError($qty);
                    // Dopo aver salvato il messaggio aggiorno la quantità da impostare a 0
                    $qty = 0;
                }


                $order_list = $this->_stock_qty->getReservedQtyOrderList($product_id);

                $stock->setQty($qty);

                if ($qty > $stock->getMinQty()){
                    $_in_stock = true;
                    $stock->setIsInStock($_in_stock);
                }
                elseif($stock->getBackorders() != \Magento\CatalogInventory\Model\Stock::BACKORDERS_NO) {
                    $_in_stock = true;
                    $stock->setIsInStock($_in_stock);
                }
                else {
                    $stock->setIsInStock($_in_stock);
                }


                $stock_data = $stock->getData();
                
                if (isset($stock_data["type_id"])) {
                    unset($stock_data["type_id"]);
                }
                
                if (isset($stock_data["stock_status_changed_automatically_flag"])) {
                    unset($stock_data["stock_status_changed_automatically_flag"]);
                }

                $this->getResource()->getConnection()->insertOnDuplicate("cataloginventory_stock_item", $stock_data);

                if ($raw_qty != $qty){
                    $preserved_qty = $raw_qty - $qty;
                    $message = sprintf("Preserved Quantity '%s' For Product ID '%s' SKU '%s' (From orders: '%s')", $preserved_qty ,$product_id, $this->getSku(),$order_list);
                    $this->error(false, $message);

                }

                $this->_logger->info('Process STOCK - qty ' . $qty . ' ID: '.$product_id . ' SKU: '.$this->getSku());


            } catch(\Exception $e){
                exit($e->getMessage());
                $this->_logger->info('Errore nel salvataggio dello Stock. ' . $e->getMessage() );
            }

        }

        return $this;
    }

    /**
     * @param $query
     * @return bool
     */
    protected function postprocessFileRows($query) {

        try {

            $query_update_child = "
                    UPDATE cataloginventory_stock_item AS cpsi
                            INNER JOIN
                        catalog_product_entity AS cpe ON cpe.entity_id = cpsi.product_id 
                    SET 
                        is_in_stock = IF (qty > min_qty OR backorders != 0 ,1 ,0)
                    WHERE
                        cpe.type_id = 'simple';
                ";

            $this->getResource()->getConnection()->query($query_update_child);

            $query_update_parent = "
                    UPDATE cataloginventory_stock_item AS cpsi
                            INNER JOIN
                        (SELECT 
                            cpe.entity_id AS parent_id,
                                IF(SUM(qty) <= 0, 0, 1) AS in_stock,
                                SUM(qty) AS qty
                        FROM
                            catalog_product_super_link AS cpsl
                        INNER JOIN catalog_product_entity AS cpe ON cpe.{$this->getEntityField()} = cpsl.parent_id
                        INNER JOIN cataloginventory_stock_item AS csi ON csi.product_id = cpsl.product_id
                        GROUP BY cpsl.parent_id) AS table_sum ON table_sum.parent_id = cpsi.product_id 
                    SET 
                        cpsi.is_in_stock = table_sum.in_stock;
                ";

            $this->getResource()->getConnection()->query($query_update_parent);

        } catch(\Exception $e){
            exit($e->getMessage());
            $this->_logger->info('Errore nel salvataggio del Post Stock Item Fix Tuning. ' . $e->getMessage());
            return false;
        }

        return true;

    }

    /**
     * @return string
     */
    protected function getEntityField() {

        $edition = $this->_productMetadata->getEdition();
        return ($edition == 'Enterprise' || $edition == 'B2B') ? 'row_id' : 'entity_id';
    }

    public function sendErrorNotifications($file) {

        // Se sono stati rilevati errori e salvati i relativi messaggi si procede con la composizione della mail
        if(!empty($this->getEmailNotifyError())){

            // Recupero gli indirizzi email dei destinatari
            $emailAddressString = $this->_scopeConfig->getValue(self::XML_PATH_EMAIL_ADDRESS_NOTIFY_ERROR);

            if($emailAddressString && $emailAddressString != '')
            {
                $emailAddress = explode(',', $emailAddressString);
                $emailAddress = array_map('trim', $emailAddress);

                $storeName = $this->_scopeConfig->getValue(self::XML_PATH_STORE_NAME);
                // Compongo il testo del messaggio con i vari errori
                $errorString = "<p>".implode("</p><p>", $this->getEmailNotifyError())."</p>";

                // Array con i dati che serviranno a comporre la mail
                $data = array(
                    'mail_subject' => 'Reporting breakdowns in stock loading for '.$storeName,
                    'file_name' => $file->getName(),
                    'errors' => $errorString
                );
                
                // Invio email di notifica
                $this->_emailnotification_helper->notify($emailAddress, $data);
            }
        }

    }


}